#include <ADuC7060.H>

//*** <<< Use Configuration Wizard in Context Menu >>> *** 
//  

///////////////////////////////////////////////////////
// Filename:    NestedVIC.c
// Author:      ADI
// Date:        26/11/2007
// Description: Demonstrates the Nested Vectored Interrupt Controller
// 
///////////////////////////////////////////////////////

// Constants
#define SW_INT                 1  
#define T0_INT                 3  
#define T1_INT                 4  
#define T2_INT                 5  
#define T3_INT                 6  
#define ADC_INT                10 
#define UART_INT               11 
#define SPIM_INT               12 
#define XIRQ0_INT              13 
#define XIRQ1_INT              14 
#define I2CM_INT               15 
#define I2CS_INT               16 
#define SPIS_INT               17 
#define XIRQ2_INT              18 
#define XIRQ3_INT              19 


#define IRQ3CLRI_BIT           19 
#define IRQ2CLRI_BIT           18 
#define IRQ1CLRI_BIT           14 
#define IRQ0CLRI_BIT           13 

// IRQCONN Bit settings
#define IRQCONN_IRQ_NEST_BIT      0
#define IRQCONN_FIQ_NEST_BIT      1

// IRQP0 MMR Bit Designations
#define IRQP0_T3PI_BIT            24
#define IRQP0_T2PI_BIT            20
#define IRQP0_T1PI_BIT            16
#define IRQP0_T0PI_BIT            12
#define IRQP0_SWIINTP_BIT         4

// IRQP1 MMR Bit Designations
#define IRQP1_I2CMPI_BIT          28
#define IRQP1_IRQ1PI_BIT          24
#define IRQP1_IRQ0PI_BIT          20
#define IRQP1_SPIMPI_BIT          16
#define IRQP1_UARTPI_BIT          12
#define IRQP1_ADCPI_BIT            8


// IRQP2 MMR Bit Designations
#define IRQP2_IRQ3PI_BIT          12
#define IRQP2_IRQ2PI_BIT          8
#define IRQP2_SPISPI_BIT          4
#define IRQP2_I2CSPI_BIT          0


#define DIR_VAL                  0xFF000000


/* Local Function Prototypes */

/* FIQ\IRQ Handler Functions */
void IRQ0_FIQ_SOURCE (void);
void IRQ1_SW_INT     (void);
void IRQ3_T0_INT     (void);
void IRQ4_T1_INT     (void);  
void IRQ5_T2_INT     (void);
void IRQ6_T3_INT     (void);
void IRQ10_ADC_INT   (void);
void IRQ11_UART_INT  (void);
void IRQ12_SPIM_INT  (void);
void IRQ13_XIRQ0_INT (void);
void IRQ14_XIRQ1_INT (void);
void IRQ15_I2CM_INT  (void);
void IRQ16_I2CS_INT  (void);
void IRQ17_SPIS_INT  (void);
void IRQ18_XIRQ2_INT (void);
void IRQ19_XIRQ3_INT (void);
void IRQx_NULL_INT   (void);


/* Exception Handlers */
void PAB_Function    (void);
void DAB_Function    (void);
void SWI_Function    (void);
void UNDEF_Function  (void);

/*    
   Define a type of function suitable for handling interrupts (and for
   inclusion in our table of interrupts)

   Requirements
   1) Does not accept paramters
   2) Does not return any value
   3) Will return normally to its caller
*/
typedef void (*tyIntHndlr)(void);  


/*    Table of Interupts Handlers 

      The VIC can handle up to 32 interrupt sources but ~20 are implemented 
      on the ADuC7060.

      The table MUST be aligned to a 128 byte boundary.
*/
const tyIntHndlr IntHndlrTable[32] __attribute__ ((aligned (128)))  = 
{
   IRQ0_FIQ_SOURCE,             /* 0  - Any FIQ active         */
   IRQ1_SW_INT    ,             /* 1  - Software               */
   IRQx_NULL_INT  ,             /* 2  - Unused                 */
   IRQ3_T0_INT    ,             /* 3  - Timer 0                */
   IRQ4_T1_INT    ,             /* 4  - Timer 1                */
   IRQ5_T2_INT    ,             /* 5  - Timer 2                */
   IRQ6_T3_INT    ,             /* 6  - Timer 3                */
   IRQx_NULL_INT  ,             /* 7  - Unused                 */
   IRQx_NULL_INT  ,             /* 8  - Unused                 */
   IRQx_NULL_INT  ,             /* 9  - Unused                 */
   IRQ10_ADC_INT  ,             /* 10 - ADC Interrupt          */
   IRQ11_UART_INT ,             /* 11 - UART interrupt         */
   IRQ12_SPIM_INT  ,            /* 12 - SPI Master interrupt   */
   IRQ13_XIRQ0_INT,             /* 13 - External interrupt 0   */
   IRQ14_XIRQ1_INT,             /* 14 - External interrupt 1   */
   IRQ15_I2CM_INT ,             /* 15 - I2C Master interrupt   */
   IRQ16_I2CS_INT ,             /* 16 - I2C Slave interrupt    */
   IRQ17_SPIS_INT ,             /* 17 - SPI Slave interrupt    */
   IRQ18_XIRQ2_INT,             /* 18 - External interrupt 2   */
   IRQ19_XIRQ3_INT,             /* 19 - External interrupt 3   */
   IRQx_NULL_INT  ,             /* 20 - Unused                 */
   IRQx_NULL_INT  ,             /* 21 - Unused                 */
   IRQx_NULL_INT  ,             /* 22 - Unused                 */
   IRQx_NULL_INT  ,             /* 23 - Unused                 */
   IRQx_NULL_INT  ,             /* 24 - Unused                 */
   IRQx_NULL_INT  ,             /* 25 - Unused                 */
   IRQx_NULL_INT  ,             /* 26 - Unused                 */
   IRQx_NULL_INT  ,             /* 27 - Unused                 */
   IRQx_NULL_INT  ,             /* 28 - Unused                 */
   IRQx_NULL_INT  ,             /* 29 - Unused                 */
   IRQx_NULL_INT  ,             /* 30 - Unused                 */
   IRQx_NULL_INT  ,             /* 31 - Unused                 */
};

static unsigned long ulLoop   = 0x0;

// Function    : Delay
// Author      : ADI
// Description : main
// Revision    : 26.11.2007 - Initial
void Delay(void)
{
   volatile int iDelay = 0x0FFFF;

   while (iDelay--);
}

// Function    : LedToggle
// Author      : ADI
// Description : Toggle the led
// Revision    : 26.11.2007 - Initial
void  LedToggle(void)
{

   // Apps EvalBoard with P1.6 to blink
   if (ulLoop%2)
     GP1SET = 0x00400000;
   else
     GP1CLR = 0x00400000;

   ulLoop++;
}

// Function    : main
// Author      : ADI
// Description : main
// Revision    : 26.11.2007 - Initial
int main(void)
{  
   unsigned long ulA,ulB, ulC;

   GP1CON = 0x0;
   GP1DAT = DIR_VAL;

   POWKEY1  = 0x1;
   POWCON0  = (POWCON0 & ~0x3);
   POWKEY2  = 0xF4;

   // Setup Exception handlers
   PABORT = PAB_Function;
   DABORT = DAB_Function;
   SWI    = SWI_Function;
   UNDEF  = UNDEF_Function;


   // Let the VIC know about our vector table
   IRQBASE           = (unsigned long)(((unsigned long)IntHndlrTable) >> 7);

   // Enable nesting and prioritisation of IRQ and FIQ interrupts
   IRQCONN           = (0x1UL << IRQCONN_IRQ_NEST_BIT) |
                       (0x1UL << IRQCONN_FIQ_NEST_BIT) ;

   // Set priorities
   IRQP0             = (0x3UL << IRQP0_T3PI_BIT)   | 
                       (0x5UL << IRQP0_T2PI_BIT)   |
                       (0x4UL << IRQP0_T1PI_BIT)   |
                       (0x2UL << IRQP0_T0PI_BIT)   |
                       (0x0UL << IRQP0_SWIINTP_BIT) ;

   IRQP1             = (0x0UL << IRQP1_I2CMPI_BIT) | 
                       (0x0UL << IRQP1_IRQ1PI_BIT) |
                       (0x1UL << IRQP1_IRQ0PI_BIT) |
                       (0x0UL << IRQP1_SPIMPI_BIT) |
                       (0x0UL << IRQP1_UARTPI_BIT) |
                       (0x0UL << IRQP1_ADCPI_BIT ) ;

   IRQP2             = (0x0UL << IRQP2_IRQ3PI_BIT) | 
                       (0x0UL << IRQP2_IRQ2PI_BIT) |
                       (0x0UL << IRQP2_SPISPI_BIT) |
                       (0x0UL << IRQP2_I2CSPI_BIT) ;

   // Configure ADC
   ADCMSKI         = 0x0001;

   ADCFLT          = 0x87;        // Chop off, 1Khz samping rate, SF = 7.
   ADC0CON         = 0x8000;  // Enable ADC0, Int reference, Gain = 1.

   ADCMDE          = 0x01; // Continuous Conversion mode, Normal mode

   // Configure external interrupts
   IRQCONE           = 0x03;        // XIRQ0 low edge triggered

   // Configure Timer 0
   T0LD            = 0x300;       // Enable Timer 1
   T0CON           = 0x4C0;       // Periodic, HCLK, div/1


   // Configure Timer 1
   T1LD            = 0x300;       // Enable Timer 1
   T1CON           = 0x4C0;       // Periodic, HCLK, div/1

   // Configure Timer 2
   T2LD            = 0x500;       
   T2CON           = 0x6C0;       

   // Configure Timer 3
   T3LD            = 0x320;       
   T3CON           = 0xC1;


   // Enable IRQs
   IRQEN           = (1UL << T1_INT)  |
                     (1UL << T2_INT)  |
                     (1UL << ADC_INT) |
                     (1UL << XIRQ0_INT);

   // Enable FIQs
   FIQEN           =    (1UL << T0_INT)  |
                        (1UL << T3_INT); 

   ulA = 0x3;
   ulB = 0x4;
   while (1)
     {
     ulC = ulA+ulB+ulC;
     // Wait for interrupts
     LedToggle();
     Delay();
     }
}

// Function    : IRQ0_FIQ_SOURCE
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ0_FIQ_SOURCE (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ1_SW_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ1_SW_INT     (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ3_T0_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ3_T0_INT     (void)
{
   T0CLRI = 0xFF;
}

// Function    : IRQ4_T1_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ4_T1_INT     (void)  
{
   // Clear the interrupt
   T1CLRI = 0xFF;
}

// Function    : IRQ5_T2_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ5_T2_INT     (void)
{
   // Clear the interrupt
   T2CLRI = 0xFF;
}

// Function    : IRQ6_T3_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ6_T3_INT     (void)
{
   // Clear the interrupt
   T3CLRI = 0xFF;
}

// Function    : IRQ10_ADC_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ10_ADC_INT   (void)
{
   // Clear the interrupt
   if(ADCSTA & 0x1)
      (void)ADC0DAT;
   if(ADCSTA & 0x2)
      (void)ADC1DAT;
}

// Function    : IRQ11_UART_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ11_UART_INT  (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ12_SPIM_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ12_SPIM_INT  (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ13_XIRQ0_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ13_XIRQ0_INT (void)
{
   // Clear an edge level interrupt
   IRQCLRE = 0x1UL << IRQ0CLRI_BIT;
}

// Function    : IRQ14_XIRQ1_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ14_XIRQ1_INT (void)
{
   // Clear an edge level interrupt
   IRQCLRE = 0x1UL << IRQ1CLRI_BIT;
}

// Function    : IRQ15_I2CM_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ15_I2CM_INT  (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ16_I2CS_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ16_I2CS_INT  (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ17_SPIS_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ17_SPIS_INT  (void)
{
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : IRQ18_XIRQ2_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ18_XIRQ2_INT (void)
{
   // Clear an edge level interrupt
   IRQCLRE = 0x1UL << IRQ2CLRI_BIT;
}

// Function    : IRQ19_XIRQ3_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQ19_XIRQ3_INT (void)
{
   // Clear an edge level interrupt
   IRQCLRE = 0x1UL << IRQ3CLRI_BIT;
}

// Function    : IRQx_NULL_INT
// Author      : ADI
// Description : Interrupt handler
// Revision    : 26.11.2007 - Initial
void IRQx_NULL_INT   (void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}


// Function    : PAB_Function
// Author      : ADI
// Description : Program abort exception handler
// Revision    : 26.11.2007 - Initial
void PAB_Function(void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}

// Function    : DAB_Function
// Author      : ADI
// Description : Data abort exception handler
// Revision    : 26.11.2007 - Initial
void DAB_Function(void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}


// Function    : SWI_Function
// Author      : ADI
// Description : SWI exception handler
// Revision    : 26.11.2007 - Initial
void SWI_Function(void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}


// Function    : UNDEF_Function
// Author      : ADI
// Description : UNDEF exception handler
// Revision    : 26.11.2007 - Initial
void UNDEF_Function(void)
{
   // This should never be called
   while(1); // Infinite loop until we provide an actual handler
}




